bluefin-internal-0.8.0.0: The Bluefin effect system, internals
Safe HaskellNone
LanguageHaskell2010

Bluefin.Internal.Vault

Synopsis

Documentation

lookup :: Key a -> Vault -> Maybe a Source #

adjust :: (a -> a) -> Key a -> Vault -> Vault Source #

insert :: Key a -> a -> Vault -> Vault Source #

newtype Key a Source #

Constructors

MkKey (Key (Any :: Type)) 

fromMine :: Key a -> Key a Source #

toMine :: Key a -> Key a Source #

type Vault = Vault RealWorld #

A persistent store for values of arbitrary types.

This variant is the simplest and creates keys in the IO monad. See the module Data.Vault.ST if you want to use it with the ST monad instead.

empty :: Vault #

The empty vault.