Bluefin.Internal.Vault
lookup :: Key a -> Vault -> Maybe a Source #
adjust :: (a -> a) -> Key a -> Vault -> Vault Source #
insert :: Key a -> a -> Vault -> Vault Source #
newtype Key a Source #
Constructors
newKey :: IO (Key a) Source #
fromMine :: Key a -> Key a Source #
toMine :: Key a -> Key a Source #
type Vault = Vault RealWorld #
A persistent store for values of arbitrary types.
This variant is the simplest and creates keys in the IO monad. See the module Data.Vault.ST if you want to use it with the ST monad instead.
IO
ST
empty :: Vault #
The empty vault.